@@ -28,7 +28,7 @@ int failure()
 int main (int argc, char *argv[])
 {
     if (argc != 5) {
-        printf("Format: %s <string|NULL> <delimiter> <position> <expected|NULL>\n", argv[0]);
+        printf("Format: %s <string> <delimiter> <position> <expected|NULL>\n", argv[0]);
         return EXIT_FAILURE;
     }
 
@@ -37,10 +37,6 @@ int main (int argc, char *argv[])
     int position = atoi(argv[3]);
     char *expected = argv[4];
 
-    if (strcmp(s, "NULL") == 0) {
-        s = NULL;
-    }
-
     char *result = duo_split_at(s, *delimiter, position);
 
     if ((result == NULL && strcmp(expected, "NULL") == 0) ||
