@@ -123,7 +123,8 @@ pam_sm_authenticate(pam_handle_t *pamh, int pam_flags,
      * without.
      */
     duopam_const char *ip, *service, *user;
-    const char *cmd, *p, *config, *host;
+    const char *cmd, *p, *host;
+    const char *config = DUO_CONF;
 
     int i, flags, pam_err, matched;
 
@@ -131,9 +132,7 @@ pam_sm_authenticate(pam_handle_t *pamh, int pam_flags,
     duo_config_default(&cfg);
 
     /* Parse configuration */
-    config = DUO_CONF;
-    duo_syslog(LOG_INFO, "Loading config file %s",
-        config);
+    duo_syslog(LOG_INFO, "Loading config file %s", config);
     if(parse_argv(&config, argc, argv) == 0) {
         return (PAM_SERVICE_ERR);
     }
@@ -154,6 +153,9 @@ pam_sm_authenticate(pam_handle_t *pamh, int pam_flags,
             !cfg.skey || !cfg.skey[0] || !cfg.ikey || !cfg.ikey[0]) {
         duo_syslog(LOG_ERR, "Missing host, ikey, or skey in %s", config);
         return (cfg.failmode == DUO_FAIL_SAFE ? PAM_SUCCESS : PAM_SERVICE_ERR);
+    } else if (cfg.autopush && cfg.verified_push) {
+        duo_syslog(LOG_ERR, "autopush and verified_push cannot both be enabled in %s", config);
+        return (cfg.failmode == DUO_FAIL_SAFE ? PAM_SUCCESS : PAM_SERVICE_ERR);
     }
 
 #ifdef OPENSSL_FIPS
@@ -272,6 +274,10 @@ pam_sm_authenticate(pam_handle_t *pamh, int pam_flags,
         flags |= DUO_FLAG_AUTO;
     }
 
+    if (cfg.verified_push) {
+        flags |= DUO_FLAG_VERIFIED_PUSH;
+    }
+
     pam_err = PAM_SERVICE_ERR;
 
     for (i = 0; i < cfg.prompts; i++) {
